package fr.insa.clavardator.client.server;

import fr.insa.clavardator.lib.network.TcpConnection;
import fr.insa.clavardator.lib.users.UserInformation;
import fr.insa.clavardator.lib.util.ErrorCallback;
import fr.insa.clavardator.lib.util.ParametrizedCallback;
import fr.insa.clavardator.lib.util.SimpleCallback;
import org.jetbrains.annotations.Nullable;

import java.util.ArrayList;

/**
 * Interface exposing public methods necessary for any presence server.
 *
 * @implNote Implement this interface when creating your own presence server class,
 * then update the {@link fr.insa.clavardator.client.server.PresenceFactory factory}
 * to add your new implementation.
 */
public interface Presence {

	/**
	 * Subscribes to this presence server notifications.
	 * A list of Ids representing the current active users is returned.
	 *
	 * @param callback Called when subscription completes
	 */
	void subscribe(ParametrizedCallback<ArrayList<UserInformation>> callback, @Nullable ErrorCallback errorCallback);

	/**
	 * Stops subscription to the presence server by closing TCP connections.
	 * This will stop notifications.
	 *
	 * @implNote Call this before exiting the app.
	 * If not, the presence server will wait for a tcp timeout before marking this user as disconnected.
	 */
	void unsubscribe(SimpleCallback callback, @Nullable ErrorCallback errorCallback);

	/**
	 * Gets a connection to the proxy.
	 * This can be used to initialize a
	 * {@link fr.insa.clavardator.client.users.PeerUser Peeruser}
	 * and send messages like on a local network.
	 *
	 * @return The server address
	 */
	TcpConnection getProxyConnection();
}
